/*
ctransitems.h
Copyright (C)2001  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001-2004 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#ifndef _CTRANSITEM_
#define _CTRANSITEM_

#include <list>
#include <algorithm>

#ifndef _BHHDR_H_
#include "pos.h"
#endif

#ifndef _BHDATETIME_H_
#include "datetime.h"
#endif 

namespace BhPos{
	
/**
 * @defgroup trans Transactions
 * @{
 *
 * @ingroup commlib
 * @{	
 */

enum BhPluType{
   BHSI_PLUTYPE_UNKNOWN,
   BHSI_PLUTYPE_EAN13,
   BHSI_PLUTYPE_UPCA,
   BHSI_PLUTYPE_ISBN,
   BHSI_PLUTYPE_MFGPROD
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhSaleItemType{
   BHSI_SITYPE_UPLU,
   BHSI_SITYPE_PLU,         //! standard sale item
   BHSI_SITYPE_PRICE,       //! unknown PLU with price
   BHSI_SITYPE_DESCPRICE,   //! unknown plu with desc/price
   BHSI_SITYPE_ALL,         //! unknown plu with full entry
   BHSI_SITYPE_RETURN,      //! return in sale
	BHSI_SITYPE_LAYBY,       //! layby in sale
   BHSI_SITYPE_DETAIL,      //! detailed
	BHSI_SITYPE_DELIVERY,    //! delivered item
	BHSI_SITYPE_UNDELIVERED, //! undelivered item
	BHSI_SITYPE_RENTAL,      //! rental item
	BHSI_SITYPE_RENTALRETURN,//! rental return item
	BHSI_SITYPE_EPHEMERAL    //! ephemeral item
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhItemDiscountType{
   BHSI_DISCOUNT_NONE=0,
   BHSI_DISCOUNT_USER,
   BHSI_DISCOUNT_PROMO,
   BHSI_DISCOUNT_PRICEBREAK,
   BHSI_DISCOUNT_CUSTOMER,
   BHSI_DISCOUNT_USERTRANS,
   BHSI_DISCOUNT_CUSTOMERTRANS,
   BHSI_DISCOUNT_PROMOTRANS
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhItemDepositType{
   BHSI_DEPOSIT_NONE=0,
   BHSI_DEPOSIT_REFUNDABLE,    //! refundable deposit
   BHSI_DEPOSIT_REDEMPTION     //! redemption deposit
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CSaleItem{
private:
   char          PLU[32];
   unsigned char plutype;
   unsigned char type;
   char          displayText[64];
   char          recuText[32];
   int           quantity;
   float         tax;
   float         discount;
   float         unitPrice;
   float         deposit;
   int           reason;
   unsigned char flags;
   char          flag_msg[64];

   BhItemDiscountType discount_type;
   BhItemDepositType  deposit_type;

protected:
   unsigned short site;
   unsigned short dept;
   unsigned short group;
   unsigned short station;
   int            user;
   BhDateTime     timestamp;

public:
   CSaleItem();
   CSaleItem(CSaleItem const&);
   ~CSaleItem();
   CSaleItem& operator=(const CSaleItem&);
   CSaleItem* operator&();
   bool operator==(CSaleItem const&);
   bool operator!=(CSaleItem const&);
   bool operator<(CSaleItem const&);
   
   /*!
		Clear to default values
	*/
   void  Clear();
   
   const int            GetType();
   const int            PluSize();
   const unsigned char  GetPluType();
   const char          *GetPLU();
   const char          *GetDispText();
   const char          *GetRecText();
   const int            GetQuantity();
   const float          GetTax();
   const float          GetDiscount();
   const float          GetTotalDiscount();
   BhItemDiscountType   GetDiscountType(){ return discount_type; }
   BhItemDepositType    GetDepositType(){ return deposit_type; }
   const float          GetPrice();
   const float          GetSubTotal();
   const float          GetTotal();
   const float          GetDeposit();
   const unsigned char  GetFlags(){ return flags; }
   const char          *GetMsg();
   const int            GetReason(){ return reason; }
   const BhDateTime&    GetTimestamp();
   void                 GetTimestamp(BhTimestamp *);
   const int            GetUser(){ return user; }
   const unsigned short GetSite(){ return site; }
   const unsigned short GetDept(){ return dept; }
   const unsigned short GetGroup(){ return group; }
   const unsigned short GetStation(){ return station; }
   
   void  SetType(int);
   void  SetPluType(unsigned char);
   bool  SetPLU(const char *);
   bool  SetDispText(char *);
   bool  SetRecText(char *);
   bool  SetQuantity(int);
   bool  SetTax(float);
   bool  SetDiscount(float);
   void  SetDiscountType(BhItemDiscountType t){ discount_type=t; }
   void  SetDepositType(BhItemDepositType t){ deposit_type=t; }
   bool  SetPrice(float);
   void  SetDeposit(float);
   void  SetFlags(unsigned char f){ flags=f; }
   void  SetMsg(char *);
   void  SetReason(int i){ reason=i; }
   void  SetTimestamp(BhDateTime&);
   void  SetUser(int i) { user=i; }
   void  SetWhere(char *);
   void  SetSite(unsigned short i) { site=i; }
   void  SetDept(unsigned short i) { dept=i; }
   void  SetGroup(unsigned short i) { group=i; }
   void  SetStation(unsigned short i) { station=i; }
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhPayType{
   PTYPE_UNKNOWN=0,
   PTYPE_CASH,
   PTYPE_CREDIT,
   PTYPE_CHEQUE,
   PTYPE_SMARTC,
   PTYPE_CREDITRETURN,
   PTYPE_CREDITSLIP,
   PTYPE_GIFTVOUCHER,
   PTYPE_TPVOUCHER,
   PTYPE_INTCOUPON,
   PTYPE_TPCOUPON,
   PTYPE_LUNCHSTAMP,
   PTYPE_INHOUSECHARGE,
   PTYPE_STORECARD,
   PTYPE_OTHER,
   PTYPE_CREDIT_CB,
   PTYPE_CHEQUE_CB,
   PTYPE_SMARTC_CB,
   PTYPE_TOKEN_CB,
   PTYPE_VOUCHER_CB,
   PTYPE_COUPON_CB,
   PTYPE_OTHER_CB
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhPayTypeFlag{
   PTYPE_FLAG_NONE=0,
   PTYPE_FLAG_SIMPLEAUTH,
   PTYPE_FLAG_PREAUTH,
   PTYPE_FLAG_DONE,
   PTYPE_FLAG_FAIL,
   PTYPE_FLAG_ERROR
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CPayItem{
private:
   BhPayType     ID;
   BhPayTypeFlag flags;
   char          name[32];
   double        amount;

protected:
   BhDateTime      timestamp;
   unsigned short  site;
   unsigned short  dept;
   unsigned short  group;
   unsigned short  station;
   int             			 user;
	long						 auth;

public:
   CPayItem();
   CPayItem(CPayItem const &);
   CPayItem(BhPayType, char *, double);
   ~CPayItem();
   
   CPayItem &operator=(CPayItem const &);
   CPayItem* operator&();
   bool operator==(CPayItem const &);
   bool operator!=(CPayItem const &);
   bool operator<(CPayItem const &);
   
   void  Clear();
   
   const double         GetAmount();
   const BhPayType      GetID();
   const BhPayTypeFlag  GetFlags();
   const char          *GetName();
   const BhDateTime&    GetTimestamp();
   void                 GetTimestamp(BhTimestamp *);
   const int            GetUser(){ return user; }
   const unsigned short GetSite(){ return site; }
   const unsigned short GetDept(){ return dept; }
   const unsigned short GetGroup(){ return group; }
   const unsigned short GetStation(){ return station; }
	const long GetAuth(){ return auth; }
   
   void  SetAmount(double);
   void  AddAmount(double);
   void  SetID(BhPayType);
   void  SetFlags(BhPayTypeFlag);
   void  SetName(char *);
   void  SetTimestamp(BhDateTime&);
   void  SetUser(int i) { user=i; }
   void  SetWhere(char *);
   void  SetSite(unsigned short i) { site=i; }
   void  SetDept(unsigned short i) { dept=i; }
   void  SetGroup(unsigned short i) { group=i; }
   void  SetStation(unsigned short i) { station=i; }
	void  SetAuth(long i) { auth=i; }
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhTransStatus{
   BHTRANS_STATUS_UNKNOWN=0,
   BHTRANS_STATUS_IDLE,
   BHTRANS_STATUS_TRANSNUM,
   BHTRANS_STATUS_OPEN,
   BHTRANS_STATUS_TENDER,
   BHTRANS_STATUS_PAYAUTH,
   BHTRANS_STATUS_COMMIT,
   BHTRANS_STATUS_OPENFREE,
   BHTRANS_STATUS_OPENPICKEDUP,
   BHTRANS_STATUS_REOPENED,
   BHTRANS_STATUS_CLOSED
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhTransNumber{
   BHNUM_SUBTOTAL,      /*!Transaction Number - subtotal*/
   BHNUM_TOTALITEMS,    /*!Transaction Number - total number of items*/
   BHNUM_TOTAL,         /*!Transaction Number - total*/
   BHNUM_DISCOUNT,      /*!Transaction Number - discount*/
   BHNUM_TAX,           /*!Transaction Number - tax*/
   BHNUM_TENDERED,      /*!Transaction Number - tendered*/
   BHNUM_CHANGE,        /*!Transaction Number - change*/
   BHNUM_TOTALTENDERED, /*!Transaction Number - total tendered*/
   BHNUM_CASHTENDERED,  /*!Transaction Number - cash tendered*/
   BHNUM_TOTALRETURN,   /*!Transaction Number - returns total*/
   BHNUM_LAYBYTOTAL,    /*!Transaction Number - laybys total*/
   BHNUM_LAYBYDEPOSIT,  /*!Transaction Number - layby deposits total*/
   BHNUM_RENTALTOTAL,   /*!Transaction Number - rentals total*/
   BHNUM_RENTALDEPOSIT, /*!Transaction Number - rentals deposit total*/
   BHNUM_ROUNDUNIT      /*!Transaction Number - currency rounding unit*/
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhTransDiscountType{
   BHTRANS_DISCOUNT_NONE=0,
   BHTRANS_DISCOUNT_USER,
   BHTRANS_DISCOUNT_PROMO,
   BHTRANS_DISCOUNT_PRICEBREAK,
   BHTRANS_DISCOUNT_CUSTOMER
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhTransRoundType{
   /*! Transaction Round Type - no currency rounding */
   BHTRANS_ROUND_NONE=0, 
   /*! Transaction Round Type - round to nearest currency rounding */   
   BHTRANS_ROUND_ROUND,
   /*! Transaction Round Type - round up currency rounding */
   BHTRANS_ROUND_UP,
   /*! Transaction Round Type - round down currency rounding */
   BHTRANS_ROUND_DOWN 
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CReturnReason 
{
private:
   int reason;
   int num;

public:
   CReturnReason();
   CReturnReason(int r, int n);
   ~CReturnReason();
   bool operator==(CReturnReason const &r);
   bool operator==(const int rid);
   bool operator!=(CReturnReason const &r);
   bool operator!=(const int tid);
   int operator++();
   int operator--();
   CReturnReason &operator=(CReturnReason const &r);
   int  GetReason() { return reason; }
   int  GetNum() { return num; }
   void SetReason(int i) { reason=i; }
   void SetNum(int i) { num=i; }
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CTransItems 
{
protected:
   BhTransStatus status;
   BhTransDiscountType discount_type;
   int    ID;
   int    nameID;
   int    numunknown;
   int    numprice;
   int    numreturn;
   int    numlayby;
   int    numrental;
   int    numephemeral;
   int    numreturnreasons;
   int    numredemption;

   BhTransRoundType  currencyRound;
   double roundvalue;
   
   double total_tendered;
   double cash_tendered;
   double cash_change;

   double discount_margin;
   double total_discount;
   double total_tax;
   double total_items;
   double sub_total;
   double total;
   
   double total_return;
   double total_rental;
   double total_layby;
   double layby_deposit;
   double rental_deposit;
   
   std::list<CSaleItem>     items;
   std::list<CPayItem>      p_items;
   std::list<CReturnReason> rr_items;

public:
   CTransItems();
   CTransItems(CTransItems const & t);
   CTransItems(const int tid);
   virtual ~CTransItems();
   bool operator==(CTransItems const &t);
   bool operator==(const int tid);
   bool operator!=(CTransItems const &t);
   bool operator!=(const int tid);
   CTransItems &operator=(CTransItems const &t);
   CTransItems *operator&();
   bool operator<(CTransItems const &t);
   int  operator+=(CTransItems& t);
   
   const BhTransStatus GetStatus(){ return status; }
   void  SetStatus(BhTransStatus s){ status=s; }
   
   int    Rationalize();
   int    UpdateAll();
   CTransItems &Updater(CSaleItem&);
   bool   Update(int n, CSaleItem& si);
   bool   Update(CSaleItem&);
   bool   Update(CPayItem&);
   double Round(double amount);
	
	const int NumAllItemsQty();
	const int NumAllItems(){ return items.size(); }
   const int NumAllDispItems(){ return items.size() + numredemption; }
	const int NumItems() { return items.size() - (numunknown + numreturn + numlayby + numrental); }
	const int NumReturnItems() { return numreturn; }
   const int NumPriceItems() { return numprice; }
	const int NumUnknownItems() { return numunknown; }
	const int NumPayment() { return p_items.size(); }
	const int NumRentalItems() { return numrental; }
	const int NumLaybyItems() { return numlayby; }
	const int NumEphemeral() { return numephemeral; }
   const int NumReturnReasons() { return numreturnreasons; }
   
   const CSaleItem& CurrItem(int seeker);
   
   /*! Finds an item by index.
		\param index the index in the list of the item to get
		\param item to be filled.
		\return 0 if item found.
	*/
   int GetItem(int index, CSaleItem& item);
   /*! Finds the first in item in a backward search.
		\param item the item to be filled, the barcode of the item should be set prior to calling this function
		\return 0 if item found.
	*/
   int GetItem(CSaleItem& item);
   int GetPayItem(int index, CPayItem& item);
   int GetPayItem(CPayItem& item);
   
   const double GetNumber(BhTransNumber);
   void         SetNumber(BhTransNumber, double);
   const int    GetID(){ return ID; }
   const int    GetNameID(){ return nameID; }
   const int    GetReturnReason();
   
   BhTransDiscountType GetDiscountType() { return discount_type; }
   void SetDiscountType(BhTransDiscountType t) { discount_type=t; }
   
   void SetID(int i){ ID=i; }
   void SetNameID(int i){ nameID=i; }
   int  Add(CSaleItem& item, bool bUpdate);
   int  Edit(CSaleItem& item);
   int  Delete(CSaleItem& item);
   int  Delete(int index);
   int  Decrement(CSaleItem& item, int *);
   int  Exists(CSaleItem& item);
   int  Exists(CPayItem& item);
   int  Pop(CSaleItem& item);
   int  PopUnknown(CSaleItem& item);
   int  PopPrice(CSaleItem& item);
   int  PopRental(CSaleItem& item);
   int  PopLayby(CSaleItem& item);
   int  PopReturn(CSaleItem& item);
   int  PopEphemeral(CSaleItem& item);
   int  Push(CSaleItem& item);
   int  PaymentExists(CPayItem& item);
   int  AddPayment(CPayItem& item, bool bUpdate);
   int  AddReturnReason(int);
   int  DeletePayment(int index);
   int  DeletePayment(CPayItem& item);
   int  RemoveReturnReason(int);
   int  PopPayment(CPayItem& item);
   void Clear();
   void ClearPayItems();
   int  StoreTransaction();
   int  RetrieveTransaction();
   void SetRoundType(BhTransRoundType t);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhItemType{
   BHITEM_TYPE_ALL,		// \enum retrieval descriptor - all
   BHITEM_TYPE_PLU,		// \enum retrieval descriptor - regular sale items
   BHITEM_TYPE_UPLU,		// \enum retrieval descriptor - unknown sale items
   BHITEM_TYPE_PRICE,	// \enum retrieval descriptor - cash sale items
   BHITEM_TYPE_RETURN,	// \enum retrieval descriptor - return sale items
   BHITEM_TYPE_LAYBY,	// \enum retrieval descriptor - layby sale items
   BHITEM_TYPE_RENTAL	// \enum retrieval descriptor - rental sale items
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CTransStack
{
protected:
   std::list<CTransItems> stack;

public:
   CTransStack();
   CTransStack(CTransStack const &);
   ~CTransStack();
   CTransStack &operator=(CTransStack const &);
   bool operator!=(CTransStack const &);
   bool operator==(CTransStack const &);
   bool operator<(CTransStack const &);
   
   const int  NumTrans(){ return stack.size(); }
   const int  NumItems(int, BhItemType);
   const int  GetNameID(int);
   
   void SetNameID(int, int);
   bool Exists(int);
   bool Update(int, CTransItems&);
   bool Get(int, CTransItems&);
   int  Add(int);
   int  AddItem(int, CSaleItem&, bool);
   int  AddPayment(int, CPayItem&, bool);
   void SetNumber(int, BhTransNumber, double);
   int  Pop(CTransItems&);
   int  Push(CTransItems&);
   void Clear(int);
   void ClearAll();
   int  Edit(CTransItems&, CTransItems&);
   int  Delete(CTransItems&);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhSSTSNumber{
   BHSSTS_NUM_ALL,
   BHSSTS_NUM_OPEN,
   BHSSTS_NUM_LISTED,
   BHSSTS_NUM_PICKEDUP,
   BHSSTS_NUM_ASSIGNEDTO
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideTrans : public CTransItems
{
protected:
   char  name[32];
   bool  bAll;
   int   creatorID;
   int   userID;
   int   stationID;
   int   targetID;
   char  targetname[32];
   int   targetNum;
   int   languageID;
   int   interfaceID;

   BhSSTSNumber sstatus;
   
public:
   CServSideTrans();
   CServSideTrans(const int tn);
   CServSideTrans(const int tn,const int pid, const int uid);
   CServSideTrans(CServSideTrans const& t);
   CServSideTrans(char *, int);
   ~CServSideTrans();
   bool operator==(CServSideTrans const& t);
   bool operator!=(CServSideTrans const& t);
   CServSideTrans &operator=(CServSideTrans const& t);
   CServSideTrans *operator&();
   int operator-=(CServSideTrans const& t);
   int operator+=(CServSideTrans const& t);

   BhSSTSNumber GetSStatus(){ return sstatus; }
   const int    GetCreatorID() { return creatorID; }
   const int    GetUserID() { return userID; }
   const char  *GetName() { return name; }
   const char  *GetTargetName() { return targetname; }

   CTransItems::GetStatus;
   CTransItems::SetStatus;
   CTransItems::Rationalize;
   CTransItems::UpdateAll;
   CTransItems::Update;
	CTransItems::NumAllItemsQty;
	CTransItems::NumAllItems;
	CTransItems::NumItems;
	CTransItems::NumReturnItems;
   CTransItems::NumPriceItems;
	CTransItems::NumUnknownItems;
	CTransItems::NumPayment;
	CTransItems::NumRentalItems;
	CTransItems::NumLaybyItems;
	CTransItems::NumEphemeral;
   CTransItems::CurrItem;
   CTransItems::GetItem;
   CTransItems::GetPayItem;
   CTransItems::GetNumber;
   CTransItems::SetNumber;
   CTransItems::GetID;
   CTransItems::GetDiscountType;
   CTransItems::SetDiscountType;
   CTransItems::SetID;
   CTransItems::Add;
   CTransItems::Edit;
   CTransItems::Delete;
   CTransItems::Decrement;
   CTransItems::Exists;
   CTransItems::Pop;
   CTransItems::PopUnknown;
   CTransItems::PopPrice;
   CTransItems::PopRental;
   CTransItems::PopLayby;
   CTransItems::PopReturn;
   CTransItems::PopEphemeral;
   CTransItems::Push;
   CTransItems::PaymentExists;
   CTransItems::AddPayment;
   CTransItems::DeletePayment;
   CTransItems::PopPayment;
   CTransItems::Clear;
   CTransItems::ClearPayItems;
   CTransItems::StoreTransaction;
   CTransItems::RetrieveTransaction;
   /*
   const int NumItems() { return items.size(); }
   const int NumUnknownItems() { return numunknown; }
   const int NumPayment() { return p_items.size(); }
   const CSaleItem& CurrItem(int seeker);
   */
};

/* }@
 *
 * @ingroup trans
 * @{
 */

enum BhTargetStatus{
   BHTARGET_STATUS_UNKNOWN=0,
   BHTARGET_STATUS_OPEN,
   BHTARGET_STATUS_RESERVED,
   BHTARGET_STATUS_INUSE,
   BHTARGET_STATUS_NEAREND
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideTarget
{
protected:
   int   ID;
   char  name[32];
   int   capacity;
   int   current;
   int   userID;
   //time_t   start_time;
   BhTargetStatus status;
   
public:
   CServSideTarget();
   CServSideTarget(const int tn);
   CServSideTarget(const int tn, const char *n, const int c);
   CServSideTarget(CServSideTarget const& t);
   ~CServSideTarget();
   bool operator==(CServSideTarget const& t);
   bool operator!=(CServSideTarget const& t);
   CServSideTarget &operator=(CServSideTarget const& t);
   int operator-=(CServSideTarget const& t);
   int operator+=(CServSideTarget const& t);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideProductOption
{
protected:
   int   ID;
   char  description[64];
   char  ReceiptText[32];
   
public:
   CServSideProductOption();
   CServSideProductOption(const int tn);
   CServSideProductOption(const int tn, const char *d, const char *r);
   CServSideProductOption(CServSideProductOption const& t);
   ~CServSideProductOption();
   bool operator==(CServSideProductOption const& t);
   bool operator!=(CServSideProductOption const& t);
   CServSideProductOption &operator=(CServSideProductOption const& t);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideProduct
{
protected:
   int   ID;
   bool  bAvailable;
   char  name[32];
   char  description[64];
   float price;
   int   printerID;
   char  ReceiptText[32];
   
   std::list<CServSideProductOption> options;

public:
   CServSideProduct();
   CServSideProduct(const int tn);
   CServSideProduct(const int t, const char *n, const char *d, const char *r,int pid, double p);
   CServSideProduct(CServSideProduct const& t);
   ~CServSideProduct();
   bool operator==(CServSideProduct const& t);
   bool operator!=(CServSideProduct const& t);
   CServSideProduct &operator=(CServSideProduct const& t);

   int AddOption(CServSideProductOption&);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideMenu
{
protected:
   int   ID;
   char  name[32];
   char  description[64];
   
   std::list<CServSideProduct> menu;

public:
   CServSideMenu();
   CServSideMenu(const int tn);
   //CServSideMenu(const int, const char *, const char *);
   CServSideMenu(CServSideMenu const& t);
   ~CServSideMenu();
   bool operator==(CServSideMenu const& t);
   bool operator!=(CServSideMenu const& t);
   CServSideMenu &operator=(CServSideMenu const& t);

   int Add(CServSideProduct&);
};

/* }@
 *
 * @ingroup trans
 * @{
 */

class CServSideTransStack
{
protected:
   std::list<CServSideTrans>  stack;
   std::list<CServSideTarget> targets;

   CServSideMenu menu;

public:
   CServSideTransStack();
   CServSideTransStack(CServSideTransStack const&);
   ~CServSideTransStack();
   /*
   bool operator==(CServSideTransStack const&);
   void operator=(CServSideTransStack const&);
   int operator-=(CServSideTransStack const&);
   int operator+=(CServSideTransStack const&);
   */
   int NumTransactions(BhSSTSNumber, int);
   //int List(BhSSTSNumber, int, char *, char *, int *);
   int List(BhSSTSNumber, int, BhTransRoverList *);
   
   const int  NumTrans(){ return stack.size(); }
   const int  NumItems(int, BhItemType);
   
   bool Exists(int);
   bool Update(int, CServSideTrans&);
   bool Get(int, CServSideTrans&);
   int  Add(int, int, int);
   int  AddItem(int, CSaleItem&, bool);
   int  AddPayment(int, CPayItem&, bool);
   void SetNumber(int, BhTransNumber, double);
   int  Pop(CServSideTrans&);
   int  Push(CServSideTrans&);
   void Clear(int);
   void ClearAll();
   int  Edit(CServSideTrans&, CServSideTrans&);
   int  Delete(CServSideTrans&);
   int  Head(CServSideTrans&);
   int  Tail(CServSideTrans&);
   int  Next(CServSideTrans&);
   int  Prev(CServSideTrans&);
   
   int  AddTarget(CServSideTarget&);
   int  AddMenu(CServSideMenu&);
   int  NumOpenTargets();
};

/* }@ 
	}@ */

}

#endif // _CTRANSITEM_
